# frozen_string_literal: true

module Hyrax
  module CrcDatasets
    class ComplexModalitiesController < ApplicationController
      before_action :set_crc_dataset
      before_action :set_complex_session, only: %i[new create]
      before_action :set_complex_modality, only: %i[show edit update]
      before_action :check_tombstone
      before_action :restrict_publication_manager
      before_action :restrict_user_to_edit_published_or_archived_work, only: [:new, :edit]
      with_themed_layout :decide_layout

      def new
        @complex_modality = ComplexModality.new
      end

      def edit
        @complex_modality = ComplexModality.find(params[:id])
      end

      def show
        @modality_members = Kaminari.paginate_array(@complex_modality.file_sets_fast_load_metadata).page(params[:page] || 1).per(params[:per_page] || 5)
        @workflow = WorkflowPresenter.new(@crc_dataset, current_ability)
      end

      def create
        @complex_modality = ComplexModality.new(complex_modality_params.merge!({ parent_source_identifier: @complex_session.source_identifier }))

        if @complex_modality.save
          update_file_set
          message = 'Complex modality was successfully created.'
          flash[:alert] = save_alert if params[:uploaded_files].present?
          redirect_to crc_dataset_complex_modality_path(@complex_modality.crc_dataset, @complex_modality), notice: message
        else
          render :new
        end
      end

      def update
        if @complex_modality.update(complex_modality_params)
          update_file_set
          message = 'Complex modality was successfully updated.'
          flash[:alert] = save_alert if params[:uploaded_files].present?
          redirect_to crc_dataset_complex_modality_path(@complex_modality.crc_dataset, @complex_modality), notice: message
        else
          render :edit
        end
      end

      private

      def set_crc_dataset
        @crc_dataset = ::CrcDataset.find(params[:crc_dataset_id])
      end

      def set_complex_session
        @complex_session = ComplexSession.find(params[:add_modalities_to_session])
      end

      def set_complex_modality
        @complex_modality = ComplexModality.find(params[:id])
      end

      def complex_modality_params
        params.require(:complex_modality).permit(:modality_title, :modality)
      end

      def decide_layout
        layout = case action_name
                 when 'show'
                   '1_column'
                 else
                   'dashboard'
                 end
        File.join(theme, layout)
      end

      def update_file_set
        if params[:uploaded_files].present? and params[:uploaded_files].any?
          uploaded_files = Hyrax::UploadedFile.find(params[:uploaded_files])
          AttachFilesToWorkJob.perform_later(@crc_dataset, uploaded_files, {for_complex_identifier: @complex_modality.source_identifier, for_complex_type: 'ComplexModality'})
        end
      end

      def save_alert
        'Your files are being processed by RDMS in the background. The metadata and access controls you specified are being applied. You may need to refresh this page to see these updates.'
      end

      def check_tombstone
        if @crc_dataset.is_tombstoned == true
          redirect_to main_app.polymorphic_path(@crc_dataset)
        end
      end

      def restrict_publication_manager
        if current_user.publication_manager?
          redirect_to main_app.polymorphic_path(@crc_dataset)
        end
      end

      def restrict_user_to_edit_published_or_archived_work
        workflow = WorkflowPresenter.new(@crc_dataset, current_ability)
        if ['published', 'archived'].include?(presenter.workflow.state)
          redirect_to main_app.polymorphic_path(@crc_dataset)
        end
      end
    end
  end
end
